/*
 * file Resource.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.ResourceList.ResponseIterator;
import javax.wvcm.WvcmException.ReasonCode;


/**
 * A proxy for a persistent resource.
 * 
 * A proxy for a persistent resource is used to retrieve information from
 * the resource and to perform operations on the resource.
 * A resource has both standard properties, which are named values whose
 * semantics are specified by the API, as well as custom properties
 * whose names are arbitrary strings selected by the client.
 * A custom property has a namespace which allows different clients
 * to use simple property names without conflicting with each other.
 * <p>
 * A proxy for a persistent resource contains the location of
 * that persistent resource on the server that maintains that resource.
 * A proxy can be created as the return value of a request to the
 * server, in which case the proxy is initialized with a set of
 * properties reflecting the state of the persistent resource at
 * the time of the request (the list of wanted
 * properties is specified as an argument to the request).
 * Alternatively, a proxy can be created locally by a client
 * (using the Provider interface), in which case it is initialized
 * with an empty set of properties.
 * <p>
 * Two proxies are equal if they are known to identify the same resource,
 * where the identity of a proxy is determined by its {@link Resource#RESOURCE_IDENTIFIER}
 * property if it has one loaded, and if not, by its {@link Resource#location}.
 * 
 * @since 1.0
 */
public interface Resource {

    /**
     * Get the location of this resource.<p>
     * The format of the location string is specific to the repository that stores
     * the persistent resource. A URL, a UNC filename, and an NFS filename
     * are examples of possible formats for a location string.
     *
     * @return the location of the persistent resource for which this
     * Resource is a proxy.
     */
    public Location location();

    /**
     * Modify the location of this resource.
     * This does not affect the value of any property of this proxy.
     */
    public void modifyLocation(Location location);

    /**
     * Get the provider of this resource.
     * 
     * @return the {@link Provider} for this Resource.
     */
    public Provider provider();

    // Resource properties

    /**
     * Get the names of standard properties available on the server resource
     * identified by this Resource.
     * 
     * @param feedback Provides optional feedback to the caller for long-running requests.
     * @return the list of names of standard properties available on the server
     * resource identified by this Resource.
     * @throws WvcmException if problems encountered communicating with server
     */
    public PropertyNameList doGetPropertyNameList(Feedback feedback)
    throws WvcmException;

    /**
     * Get the names of properties in a given namespace available
     * on the server resource identified by this Resource.
     * 
     * @param namespace the namespace for the requested property names.
     * @param feedback Provides optional feedback to the caller for long-running requests.
     * @return the list of names of properties in the specified namespace
     * available on the server resource identified by this Resource.
     * If the namespace is null, the system property names are listed.
     * If the namespace is not null, the user-defined properties
     * in that namespace are listed.
     * @throws WvcmException if problems encountered communicating with server
     */
    public PropertyNameList doGetPropertyNameList(String namespace, Feedback feedback)
    throws WvcmException;

    /**
     * Get a set of properties of this Resource from the server.
     * Like all doXyz methods, any modified properties in this Resource
     * are first written to the resource before the properties are read
     * from the resource.  Note that this makes doReadProperties semantically
     * identical to {@link #doWriteProperties}.
     * 
     * @param feedback the properties to be available in the returned proxy,
     *  and any other feedback desired, such as progress indications.
     * @return a Resource proxy for this resource containing the wanted properties.
     * A requested property named XYZ can be retrieved from
     * the resource with the <code>getXyz</code> method, when a
     * <code>getXyz</code> method is defined in the interface.
     * @throws WvcmException if problems encountered communicating with server or
     * if modified properties cannot be written (see {@link #doWriteProperties}. 
     */
    public Resource doReadProperties(Feedback feedback) throws WvcmException;

    /**
     * Get the names of properties available on this client proxy.
     * A property is available if it would not throw an exception if
     * retrieved from this proxy.
     * One difference between this and wantedPropertyRequestList
     * is that {@link #ALL_CUSTOM_PROPERTIES} would appear in wantedPropertyRequestList,
     * but the actual property names would appear in propertyNameList.
     * In addition, failing to retrieve a property would cause a property
     * to not appear in propertyNameList, 
     * setting a property would add that property to propertyNameList,
     * and forgetting or removing a property would remove that property
     * from propertyNameList.
     * @return the list of names of properties available on this client proxy.
     */
    public PropertyNameList propertyNameList();

    /**
     * Get a list of modified properties that have been updated in the proxy
     * by setXyz operations, but not been successfully written to the
     * corresponding persistent resource by a doXyz operation.
     * @return the names of properties that have been updated in
     * the proxy, but the updates have not yet been successfully
     * applied to the resource. 
     */
    public PropertyNameList updatedPropertyNameList();

    /**
     * Persists property changes to this Resource.
     * An exception is thrown if any of properties cannot be written.
     * If an exception is thrown, the properties that were not be written
     * can be determined from {@link #updatedPropertyNameList}
     * on this Resource.
     * Note that since doWriteProperties returns a new proxy whose properties are
     * specified in the feedback argument, doWriteProperties is semantically identical
     * to {@link #doReadProperties}.
     * <p>
     * Postconditions:
     * <li>(auto-checkout-controlled-property): If the property is controlled,
     *  and if this resource was a checked-in controlled,
     *  resource, this resource MUST have been checked out by this request.
     * 
     * @param feedback Specifies optional feedback to the caller.
     * @return a new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#PROPERTY_OVERWRITE_FORBIDDEN}:
     * This property has been updated by another client after this proxy was created.
     * <li>{@link ReasonCode#CANNOT_MODIFY_PROTECTED_PROPERTY}:
     * Some properties cannot be directly updated.
     * <li>{@link ReasonCode#CANNOT_MODIFY_VERSION}:
     *  If this proxy identifies a version, the request MUST fail
     *  unless the property is explicitly defined to be modifiable on a 
     *  version.
     * <li>Since this method may cause a checkout to be attempted on this resource,
     *  see {@link ControllableResource#doCheckin} for additional reason codes.
     */
    public Resource doWriteProperties(Feedback feedback) throws WvcmException;

    /**
     * Get both the content and selected properties of this resource.
     * 
     * @param content the resource content is written to this
     * byte stream.  The stream is then closed.
     * @param feedback Specifies optional feedback to the caller.
     * @return a new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#METHOD_NOT_SUPPORTED}:
     * This resource has no content.
     */
    public Resource doReadContent(
            OutputStream content, 
            Feedback feedback)
    throws WvcmException;

    /**
     * Persists content changes to a resource.
     * <p>
     * Postconditions:
     * <li>(auto-checkout-controlled-content): If this resource was a checked-in controlled,
     *  resource, this resource MUST have been checked out by this request.
     * 
     * @param content if <code>contentIdentifier</code> matches the current
     * content identifier of the persistent resource,
     * the content of the resource is replaced with the
     * bytes read from <code>content</code>, and <code>content</code>
     * is then closed.</p>
     * @param contentIdentifier if this {@link #getContentIdentifier content identifier}
     *  does not match
     * the content identifier of the persistent resource, the persistent
     * resource is left unaltered, and a WvcmException is thrown.
     * <p>
     * If reading from the stream throws a <code>java.io.IOException</code>,
     * then no further data will be read from the stream,
     * and after attempting to close the stream, a <code>WvcmException</code>
     * wrapping the <code>IOException</code> is thrown,
     * possibly leading to incomplete data being stored on the resource.</p>
     * @param feedback Specifies optional feedback to the caller.
     * @return a new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#CANNOT_MODIFY_VERSION}:
     *  If this proxy identifies a version, the request MUST fail.
     * <li>Since this method may cause a checkout to be attempted on this resource,
     *  see {@link ControllableResource#doCheckin} for additional reason codes.
     */
    public Resource doWriteContent(InputStream content, String contentIdentifier, Feedback feedback)
    throws WvcmException;

    /** Boolean flags for the doCopy method */
    public static enum CopyFlag 
    {
        /** 
         * Indicates whether to overwrite an existing binding in the destination folder
         */
        OVERWRITE("overwrite"); //$NON-NLS-1$

        private CopyFlag(String flagName) { _flagName = flagName; }

        /**
         * Returns a string representation of this flag suitable for diagnostics.
         */
        @Override
        public String toString() { return _flagName; }

        /**
         * The string form of this flag.
         */
        private final String _flagName;

    }

    /**
     * Create a copy of the resource identified by this Resource
     * at the location identified by the <code>destination</code>.
     * The content of the copy is the same as the content of the
     * resource identified by this Resource, but the properties of the
     * copy are the default properties for a new resource.
     * <p>
     * Postconditions:
     * <li>(must-not-copy-resource-identifier): The copy must be a different resource.
     *  In particular, it must have a different resource identifier.
     * <li>(copy-creates-new-resource): If the source of a doCopy is a controlled resource
     *  or a version,
     *  and if there is no resource at the destination of the doCopy,
     *  then the doCopy creates a new uncontrolled resource at the destination of the doCopy.
     * 
     * @param destination the location of the new resource created by doCopy.
     * @param flags Array of boolean flags (may be null):
     * <li>{@link CopyFlag#OVERWRITE}:
     *  Indicates whether an existing destination resource
     *  will be overwritten rather than the request failing.
     * @param feedback Specifies optional feedback to the caller.
     * @return a new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#CANNOT_OVERWRITE}:
     *  If there already is a resource at the destination,
     *  and the <code>overwrite</code> is <code>false</code>, the request MUST fail.
     * <li>{@link ReasonCode#METHOD_NOT_SUPPORTED}:
     *  This proxy MUST NOT identify a folder version or a version history.
     *  In order to create another version history whose versions have the same content,
     *  the appropriate sequence of doVersionControl, doCheckout, doWriteContent, and doCheckin requests must be made.
     */
    public Resource doCopy(Location destination, CopyFlag[] flags, Feedback feedback)
    throws WvcmException;

    /**
     * Unbinds the resource identified by this Resource
     * from all folders in its {@link #PARENT_LIST}.
     * 
     * See {@link Folder#doUnbindChild}.
     */
    public void doUnbindAll(Feedback feedback) throws WvcmException;

    /**
     * Find a persistent resource that "matches" this proxy.
     * For example, 
     * <pre>
     *   Activity activityExample = WorkspaceProvider.activity(WorkspaceProvider.location("/my-act-repo")):
     *   activityExample.setCreatorDisplayName("Geoff Clemm");
     *   Activity activity = (Activity)activityExample.doFind(null);
     * </pre> 
     * would return an activity in "/my-act-repo" created by "Geoff Clemm".
     * Unlike most doXyz methods, this method does not save changes to the proxy 
     * before executing the query. In particular, it is valid to execute 
     * this method on a proxy that could not be saved by {@link #doWriteProperties}.
     * <p>The precise rules for matching are as follows:
     * <li>A proxy Q matches a resource R if the type of Q "matches" the type of R,
     *  if the location of Q "matches" the location of R,
     *  and if for each loaded property P of the proxy Q,
     *  Q.P.value (the value of the property P of the proxy Q) "matches" R.P.value
     *  (the value of the property P of the resource R).
     *  Note that a property is loaded into a proxy as the result of reading properties values
     *  from the repository when creating the proxy (e.g., {@link #doReadProperties})
     *  or by explicitly setting property values (e.g., {@link #setComment}
     *  or more generally, {@link #setProperty}).
     * <li>The type of a proxy matches the type of a resource if the resource implements the
     *  result type of the method used to create the proxy.  So for example, if the proxy was
     *  created by {@link WorkspaceProvider#workspace}, to match that proxy a resource
     *  must implement the {@link Workspace} interface.
     * <li>The location of a proxy Q matches the location of a resource R
     *  if the resource identified by the location of Q would be in the same "repository" as R.
     *  Note that whether or not a resource would be in the same repository as another
     *  resource is decided by the implementation of the {@link #doFind} method. 
     * <li>If Q.P.value is not a list and R.P.value is a list, then Q.P.value matches
     *  R.P.value if Q.P.value matches at least one object in R.P.value.
     * <li>If Q.P.value is a list of proxies, and R.P.value is not a list, then Q.P.value matches
     *  R.P.value if at least one proxy in Q.P.value matches R.P.value.
     * <li>If Q.P.value is a list of proxies, and R.P.value is a list, then Q.P.value matches
     *  R.P.value if at least one proxy in Q.P.value matches at least one object in R.P.value.
     * <li>If Q.P is Resource.PATHNAME_LOCATION, then Q matches R if R is the resource
     *  identified by Q.P.value.
     * <li>Otherwise, Q.P.value matches R.P.value if Q.P.value {@link Object#equals} R.P.value.
     * 
     * @param feedback the properties to be available in the returned resources.
     * @return a proxy for a persistent resource that matches this Resource.
     * If none found, <code>null</code> is returned.
     * @throws WvcmException if problems encountered communicating with server
     * or if the request could not be executed.
     */
    public Resource doFind(Feedback feedback)
    throws WvcmException;    


    /**
     * Find all persistent resources that "match" this proxy.
     * For example, 
     * <pre>
     *   Activity activityExample = WorkspaceProvider.activity(WorkspaceProvider.location("/my-act-repo")):
     *   activityExample.setCreatorDisplayName("Geoff Clemm");
     *   ResponseIterator myActivities = activityExample.doFindAll(null);
     * </pre> 
     * would return all activities in "/my-act-repo" created by "Geoff Clemm".
     * See {@link #doFind}.
     * 
     * @param feedback the properties to be available in the returned resources.
     * @return proxies for persistent resources that that match this Resource.
     * @throws WvcmException if problems encountered communicating with server
     * or if the request could not be executed.
     */
    public <T extends Resource> ResponseIterator<T> doFindAll(Feedback feedback)
    throws WvcmException;    


//  Resource Properties

    /**
     * A list of folders that contain workspaces that are being managed
     * by the same server that is managing this resource.
     */
    public static final PropertyName<ResourceList<Folder>> WORKSPACE_FOLDER_LIST =
        new PropertyName<ResourceList<Folder>>("workspace-folder-list"); //$NON-NLS-1$

    /**
     * Get the {@link #WORKSPACE_FOLDER_LIST} property.
     * 
     * @return the {@link #WORKSPACE_FOLDER_LIST} property.
     * @throws WvcmException if this Resource was not created with
     * {@link #WORKSPACE_FOLDER_LIST} as a wanted property.
     */
    public ResourceList<Folder> getWorkspaceFolderList() throws WvcmException;

    /**
     * A list of names of alternative providers for the
     * persistent resource identified by this resource.
     * Provider names are class names, and {@link Provider} objects
     * can be generated from provider names using {@link ProviderFactory#createProvider(String,ProviderFactory.Callback)}.
     * The preferred providers are specified earlier in the list.
     */
    public static final PropertyName<List<String>> PROVIDER_LIST =
        new PropertyName<List<String>>("provider-list"); //$NON-NLS-1$

    /**
     * Get the {@link #PROVIDER_LIST} property.
     * 
     * @return the {@link #PROVIDER_LIST} property.
     * @throws WvcmException if this Resource was not created with
     * {@link #PROVIDER_LIST} as a wanted property.
     */
    public List<String> getProviderList()
    throws WvcmException;

    /**
     * A resource identifier is an optional unique and immutable identifier for a resource.
     * The resource identifier may be used to test whether two proxies
     * identify the same persistent resource.
     * The resource identifier is the string form of a stable location for the resource,
     * where a stable location is a location that is not affected by moves/renames
     * by {@link Folder#doRebindChild} or by the addition/removal of additional locations for the
     * resource by {@link Folder#doBindChild} and {@link Folder#doUnbindChild}.
     * <p>
     * If {@link Provider#location} can be applied to the resource identifier of a resource
     * to produces a valid location, then that location MUST identify that resource.
     */
    public static final PropertyName<String> RESOURCE_IDENTIFIER =
        new PropertyName<String>("resource-id"); //$NON-NLS-1$

    /**
     * Get the {@link #RESOURCE_IDENTIFIER} property.
     * 
     * @return the {@link #RESOURCE_IDENTIFIER} property.
     * @throws WvcmException if this Resource was not created with
     * {@link #RESOURCE_IDENTIFIER} as a wanted property.
     */
    public String getResourceIdentifier()
    throws WvcmException;

    /**
     * A location for this resource to which {@link Location#parent} can be applied
     * zero or more times to produce a location for a folder in either
     * the {@link Resource#WORKSPACE_FOLDER_LIST} of this resource, or for a folder in
     * the {@link Workspace#ACTIVITY_FOLDER_LIST} or {@link Workspace#VERSION_HISTORY_FOLDER_LIST}
     * of some workspace that is a member of a folder in {@link Resource#WORKSPACE_FOLDER_LIST}.
     */
    public static final PropertyName<Location> PATHNAME_LOCATION =
        new PropertyName<Location>("pathname-location"); //$NON-NLS-1$

    /**
     * Get the {@link #PATHNAME_LOCATION} property.
     * 
     * @return the {@link #PATHNAME_LOCATION} property.
     * @throws WvcmException if this Resource was not created with
     * {@link #PATHNAME_LOCATION} as a wanted property.
     */
    public Location getPathnameLocation() throws WvcmException;

    /**
     * The PARENT_LIST property enables
     * clients to discover what folders contain a binding to this resource
     * (that is, which folders have this resource as a bound member).
     * The value of the PARENT_LIST property is a list of 
     * {@link Folder} objects.
     */
    public static final PropertyName<ResourceList<Folder>> PARENT_LIST =
        new PropertyName<ResourceList<Folder>>("parent-list"); //$NON-NLS-1$

    /**
     * Get the {@link #PARENT_LIST} property.
     * 
     * @return the {@link #PARENT_LIST} property.
     * @throws WvcmException if this Resource was not created with
     * {@link #PARENT_LIST} as a wanted property.
     */
    public ResourceList<Folder> getParentList()
    throws WvcmException;

    /**
     * Contains a description of the resource that is suitable
     * for presentation to a user in a tree display.
     * <p>
     * Note that the {@link #DISPLAY_NAME} of a resource is in general
     * not the binding name of that resource in its parent folder,
     * because at a given time, a resource can be bound into
     * multiple parent folders, and have a different binding name in
     * each parent folder. </p>
     */
    public static final PropertyName<String> DISPLAY_NAME =
        new PropertyName<String>("display-name"); //$NON-NLS-1$

    /**
     * Get the {@link #DISPLAY_NAME} property.
     * 
     * @return the {@link #DISPLAY_NAME} property.
     * @throws WvcmException if this property was not set and
     * this Resource was not created with
     * {@link #DISPLAY_NAME} as a wanted property.
     * @see #setDisplayName
     */
    public String getDisplayName() throws WvcmException;

    /**
     * Set the {@link #DISPLAY_NAME} property.
     * 
     * @param val the new display name to apply to the resource.
     * @see #getDisplayName
     */
    public void setDisplayName(String val);

    /**
     * A brief comment about a resource that
     * is suitable for presentation to a user.  The comment of a version
     * can be used to indicate why that version was created.
     */
    public static final PropertyName<String> COMMENT =
        new PropertyName<String>("comment"); //$NON-NLS-1$

    /**
     * Get the {@link #COMMENT} property.
     * @return the {@link #COMMENT} property.
     * @throws WvcmException if this property was not set and
     * this Resource was not created with
     * {@link #COMMENT} as a wanted property.
     * @see #setComment
     */
    public String getComment() throws WvcmException;

    /**
     * Set the {@link #COMMENT} property.
     * 
     * @param comment the new comment to apply to this Resource.
     * @see #getComment
     */
    public void setComment(String comment);

    /**
     * This property contains a description of the creator of the resource
     * that is suitable for presentation to a user.  The {@link #CREATOR_DISPLAY_NAME}
     * of a version can be used to indicate who created that version.
     * A server may automatically set the {@link #CREATOR_DISPLAY_NAME} property
     * and not allow it to be changed by a client.
     */
    public static final PropertyName<String> CREATOR_DISPLAY_NAME =
        new PropertyName<String>("creator-display-name"); //$NON-NLS-1$

    /**
     * Get the {@link #CREATOR_DISPLAY_NAME} property.
     * 
     * @return the {@link #CREATOR_DISPLAY_NAME} property.
     * @throws WvcmException if this property was not set and
     * this Resource was not created with
     * {@link #CREATOR_DISPLAY_NAME} as a wanted property.
     * @see #setCreatorDisplayName
     */
    public String getCreatorDisplayName() throws WvcmException;

    /**
     * Set the {@link #CREATOR_DISPLAY_NAME} property.
     * 
     * @param val the new creator display name to apply to the resource.
     * @see #getCreatorDisplayName
     */
    public void setCreatorDisplayName(String val);

    /** The date and time the resource was created. */
    public static final PropertyName<Date> CREATION_DATE =
        new PropertyName<Date>("creation-date"); //$NON-NLS-1$

    /**
     * Get the {@link #CREATION_DATE} property.
     * 
     * @return the {@link #CREATION_DATE} property.
     * @throws WvcmException if this Resource was not created with
     * {@link #CREATION_DATE} as a wanted property
     */
    public Date getCreationDate() throws WvcmException;

    /**
     * The date and time the content of the resource was last modified.
     */
    public static final PropertyName<Date> LAST_MODIFIED =
        new PropertyName<Date>("last-modified"); //$NON-NLS-1$

    /**
     * Get the {@link #LAST_MODIFIED} property.
     * 
     * @return the {@link #LAST_MODIFIED} property.
     * @throws WvcmException if this Resource was not created with
     * {@link #LAST_MODIFIED} as a wanted property.
     */
    public Date getLastModified() throws WvcmException;

    /**
     * The name of the character set of the resource content.
     * Character sets names must be registered; see RFC 2278 and 1700
     */
    public static final PropertyName<String> CONTENT_CHARACTER_SET =
        new PropertyName<String>("content-character-set"); //$NON-NLS-1$

    /**
     * Get the {@link #CONTENT_CHARACTER_SET} property.
     * @return the {@link #CONTENT_CHARACTER_SET} property.
     * @throws WvcmException if this property was not set and
     * this Resource was not created with
     * {@link #CONTENT_CHARACTER_SET} as a wanted property.
     * @see #setContentCharacterSet
     */
    public String getContentCharacterSet() throws WvcmException;

    /**
     * Set the {@link #CONTENT_CHARACTER_SET} property.
     * 
     * @param contentCharacterSet the content character set for the resource.
     * @see #getContentCharacterSet
     */
    public void setContentCharacterSet(String contentCharacterSet);

    /**
     * An implementation-defined String identifying the value of the content of a resource.
     * The semantics of a Content Identifier are the same as that of an HTTP ETag (see RFC 2616).
     */
    public static final PropertyName<String> CONTENT_IDENTIFIER =
        new PropertyName<String>("content-identifier"); //$NON-NLS-1$

    /**
     * Get the {@link #CONTENT_IDENTIFIER} property.
     * @return the {@link #CONTENT_IDENTIFIER} property.
     * @throws WvcmException if this Resource was not created with
     * {@link #CONTENT_IDENTIFIER} as a wanted property.
     */
    public String getContentIdentifier() throws WvcmException;

    /**
     * The name of the natural language used in a resource content,
     * represented by a {@link java.util.Locale Locale} object,
     * with country, language, and variant if appropriate.
     */
    public static final PropertyName<Locale> CONTENT_LANGUAGE =
        new PropertyName<Locale>("content-language"); //$NON-NLS-1$

    /**
     * Get the {@link #CONTENT_LANGUAGE} property.
     * @return the {@link #CONTENT_LANGUAGE} property.
     * @throws WvcmException if this property was not set and
     * this Resource was not created with
     * {@link #CONTENT_LANGUAGE} as a wanted property.
     * @see #setContentLanguage
     */
    public Locale getContentLanguage() throws WvcmException;

    /**
     * Set the {@link #CONTENT_LANGUAGE} property.
     * 
     * @param contentLanguage the content language for the resource.
     * @see #getContentLanguage
     */
    public void setContentLanguage(Locale contentLanguage);

    /** The size in bytes of the content of a resource. */
    public static final PropertyName<Long> CONTENT_LENGTH =
        new PropertyName<Long>("content-length"); //$NON-NLS-1$

    /**
     * Get the {@link #CONTENT_LENGTH} property.
     * 
     * @return the {@link #CONTENT_LENGTH} property.
     * @throws WvcmException if this Resource was not created with
     * {@link #CONTENT_LENGTH} as a wanted property.
     */
    public long getContentLength() throws WvcmException;

    /**
     * A MIME type string (see RFC 1590) indicating the media type of the
     * resource content.
     */
    public static final PropertyName<String> CONTENT_TYPE =
        new PropertyName<String>("content-type"); //$NON-NLS-1$

    /**
     * Get the {@link #CONTENT_TYPE} property.
     * A server may refuse to allow the {@link #CONTENT_TYPE} of a resource to be changed
     * after the resource has been created.
     * 
     * @return the {@link #CONTENT_TYPE} property.
     * @throws WvcmException if this property was not set and
     * this Resource was not created with
     * {@link #CONTENT_TYPE} as a wanted property.
     * @see #setContentType
     */
    public String getContentType() throws WvcmException;

    /**
     * Set the {@link #CONTENT_TYPE} property.
     * 
     * @param contentType the content type for the resource.
     * @see #getContentType
     */
    public void setContentType(String contentType);

    /**
     * Whether the content is executable (i.e, can be run as a program).
     */
    public static final PropertyName<Boolean> IS_EXECUTABLE =
        new PropertyName<Boolean>("is-executable"); //$NON-NLS-1$

    /**
     * Get the {@link #IS_EXECUTABLE} property.
     * 
     * @return the {@link #IS_EXECUTABLE} property.
     * @throws WvcmException if this property was not set and
     * this Resource was not created with
     * {@link #IS_EXECUTABLE} as a wanted property.
     * @see #setIsExecutable
     */
    public Boolean getIsExecutable() throws WvcmException;

    /**
     * Set the {@link #IS_EXECUTABLE} property.
     * 
     * @param isExecutable whether the resource is executable.
     * @see #getIsExecutable
     */
    public void setIsExecutable(Boolean isExecutable);

    /** 
     * A generic PropertyName that selects all custom properties
     * (i.e., properties with a non-null namespace)
     * that are defined on the resource.
     */
    public static final PropertyName<Object> ALL_CUSTOM_PROPERTIES =
        new PropertyName<Object>("all-custom-properties"); //$NON-NLS-1$

    /**
     * Get the value of the specified property of this Resource.
     * 
     * @param name the name of the property.
     * @return the value of the specified property of this Resource.
     * @throws WvcmException if this property was not set and
     * this Resource was not created with
     * the specified property as a wanted property.
     * @see #setProperty
     */
    public <T> T getProperty(PropertyName<T> name)
    throws WvcmException;

    /**
     * Lookup the value of the specified property of this Resource.
     * 
     * @param name the name of the property.
     * @return the value of the specified property of this Resource,
     *  or a WvcmException if there was an error computing the property value
     *  or if this property was not set and this resource was not created
     *  with the specified property as a wanted property.
     * @see #getProperty
     */
    public Object lookupProperty(PropertyName<?> name);

    /**
     * Adds or replaces the value of the specified property of this Resource.
     * 
     * @param name the name of the property.
     * @param value the new value of the specified property.
     * @see #getProperty
     */
    public <T> void setProperty(PropertyName<T> name, T value);

    /**
     * Initializes the value of the specified property of this Resource.
     * If that property of the Resource is non-null when it is written
     * (e.g., by a {@link #doWriteProperties} request),
     * {@link WvcmException} with {@link ReasonCode#CANNOT_OVERWRITE}
     * is thrown and the property of the Resource is not changed.
     *  
     * @param name the name of the property.
     * @param value the value of the specified property.
     * @see #getProperty
     */
    public <T> void initProperty(PropertyName<T> name, T value);

    /**
     * Adds and removes the specified values from the current value
     * of the specified list-valued property.
     * 
     * @param name the name of the property.
     * @param additions the values to be added to the value of the specified property value.
     * @param removals the values to be removed from the value of the specified property.
     * @see #setProperty
     */
    public <V, U extends Collection<V>> void setProperty(PropertyName<U> name, U additions, U removals);

    /**
     * Removes the specified property of this Resource.
     * This is just a local change to this proxy, and is not
     * persisted back to the resource identified by this Resource.
     * This is commonly used to clear an updated property that
     * caused {@link #doWriteProperties} to fail.
     * 
     * @param name the name of the property.
     */
    public void forgetProperty(PropertyName<?> name);

    /**
     * Removes the specified property of this Resource.
     * The next doXyz() request will cause the server to remove this property
     * from the resource on the server identified by this Resource.
     * 
     * @param name the name of the property.
     */
    public void removeProperty(PropertyName<?> name);

}
